"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint no-console: off , no-process-exit: off*/
require("source-map-support/register");
const util_1 = require("util");
const chalk_1 = __importDefault(require("chalk"));
const model_1 = require("./model/model");
const view_1 = require("./view/view");
const node_opcua_client_1 = require("node-opcua-client");
const make_certificate_1 = require("./make_certificate");
const check = require("check-node-version");
function check_nodejs() {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const result = yield (0, util_1.promisify)(check)({ node: ">=12" });
            if (result.isSatisfied) {
                return;
            }
            console.error("Some package version(s) failed!");
            for (const packageName of Object.keys(result.versions)) {
                if (!result.versions[packageName].isSatisfied) {
                    console.error(`Incorrect ${packageName} version. 
        your version            : ${result.versions[packageName].version.version}
        expected minimum version: ${result.versions[packageName].wanted.range}`);
                    // ${JSON.stringify(result.versions[packageName],null, " ")}
                }
            }
            process.exit();
        }
        catch (err) {
            console.error(err);
            process.exit();
        }
    });
}
// xx const updateNotifier = require("update-notifier");
const pkg = require("../package.json");
const argv = require("yargs")
    .wrap(132)
    .demand("endpoint")
    .string("endpoint")
    .describe("endpoint", "the end point to connect to ")
    .string("securityMode")
    .describe("securityMode", "the security mode")
    .string("securityPolicy")
    .describe("securityPolicy", "the policy mode")
    .string("userName")
    .describe("userName", "specify the user name of a UserNameIdentityToken ")
    .string("password")
    .describe("password", "specify the password of a UserNameIdentityToken")
    .string("node")
    .describe("node", "the nodeId of the value to monitor")
    .string("history")
    .describe("history", "make an historical read")
    .string("userCertificate")
    .describe("userCertificate", "X509 user certificate (PEM format)")
    .string("userCertificatePrivateKey")
    .describe("userCertificatePrivateKey", "X509 private key associated with the user certificate")
    .boolean("verbose")
    .describe("verbose", "display extra information")
    .alias("e", "endpoint")
    .alias("s", "securityMode")
    .alias("P", "securityPolicy")
    .alias("u", "userName")
    .alias("p", "password")
    .alias("n", "node")
    .alias("t", "timeout")
    .alias("v", "verbose")
    .alias("c", "userCertificate")
    .alias("x", "userCertificatePrivateKey")
    .example("opcua-commander  --endpoint opc.tcp://localhost:49230 -P=Basic256 -s=Sign")
    .example("opcua-commander  -e opc.tcp://localhost:49230 -P=Basic256 -s=Sign -u JoeDoe -p P@338@rd ")
    .example('opcua-commander  --endpoint opc.tcp://localhost:49230  -n="ns=0;i=2258"').argv;
const securityMode = node_opcua_client_1.MessageSecurityMode[argv.securityMode || "None"];
if (!securityMode) {
    throw new Error(`Invalid Security mode , was  ${chalk_1.default.magenta(argv.securityMode)}\nshould be  ${chalk_1.default.cyan(Object.values(node_opcua_client_1.MessageSecurityMode).filter(isNaN).join(","))}`);
}
const securityPolicy = node_opcua_client_1.SecurityPolicy[argv.securityPolicy || "None"];
if (!securityPolicy) {
    throw new Error(`Invalid securityPolicy\nwas       : ${chalk_1.default.magenta(argv.securityPolicy)}\nshould be : ${chalk_1.default.cyan(Object.keys(node_opcua_client_1.SecurityPolicy).filter((k) => typeof k === "string" && k !== "Invalid" && !k.match(/PubSub/)))}`);
}
const endpointUrl = argv.endpoint || "opc.tcp://localhost:26543";
const yargs = require("yargs");
if (!endpointUrl) {
    yargs.showHelp();
    // xx updateNotifier({ pkg }).notify();
    process.exit(0);
}
(() => __awaiter(void 0, void 0, void 0, function* () {
    yield check_nodejs();
    const { certificateFile, clientCertificateManager, applicationUri, applicationName } = yield (0, make_certificate_1.makeCertificate)();
    const model = new model_1.Model();
    const view = new view_1.View(model);
    yield model.initialize(endpointUrl, securityMode, securityPolicy, certificateFile, clientCertificateManager, applicationName, applicationUri);
    const version = require("../package.json").version;
    console.log(chalk_1.default.green(" Welcome to Node-OPCUA Commander ") + version);
    console.log(chalk_1.default.cyan("   endpoint url    = "), endpointUrl.toString());
    console.log(chalk_1.default.cyan("   securityMode    = "), node_opcua_client_1.MessageSecurityMode[securityMode]);
    console.log(chalk_1.default.cyan("   securityPolicy  = "), securityPolicy.toString());
    console.log(chalk_1.default.cyan("   certificate file = "), certificateFile);
    console.log(chalk_1.default.cyan("   trusted certificate folder = "), clientCertificateManager.trustedFolder);
    const userIdentity = (0, model_1.makeUserIdentity)(argv);
    model.doConnect(endpointUrl, userIdentity);
    model.on("connectionError", (err) => {
        console.log(chalk_1.default.red("  exiting"));
        view.logWindow.focus();
        setTimeout(() => process.exit(-1), 10000);
    });
}))();
//# sourceMappingURL=index.js.map